/* FILE: energy.h                 -*-Mode: c++-*-
 *
 * Abstract energy class, derived from Oxs_Ext class.
 *
 */

#ifndef _OXS_ENERGY
#define _OXS_ENERGY

#include "ext.h"
#include "simstate.h"
#include "key.h"
#include "meshvalue.h"
#include "output.h"
#include "util.h"

/* End includes */

/* Optional #ifdef's:
 *   #define EXPORT_CALC_COUNT to enable
 *      Oxs_ScalarOutput<Oxs_Energy> calc_count_output;
 */
#ifndef NDEBUG
# define EXPORT_CALC_COUNT 1
#endif

#ifndef REPORT_TIME
# define REPORT_TIME 0
#endif

struct Oxs_EnergyData {
public:
  const UINT4m state_id;

  Oxs_MeshValue<REAL8m>* energy_buffer;
  Oxs_MeshValue<ThreeVector>* field_buffer;

  Oxs_WriteOnceObject<REAL8m> energy_sum;
  Oxs_WriteOnceObject<const Oxs_MeshValue<REAL8m>* > energy;
  Oxs_WriteOnceObject<const Oxs_MeshValue<ThreeVector>* > field;

  Oxs_WriteOnceObject<REAL8m> pE_pt;

  Oxs_EnergyData(const Oxs_SimState& state)
    : state_id(state.Id()), energy_buffer(NULL), field_buffer(NULL) {}

private:
  // The following are left undefined on purpose:
  Oxs_EnergyData();
  Oxs_EnergyData(const Oxs_EnergyData&);
  const Oxs_EnergyData& operator=(const Oxs_EnergyData&);
};

class Oxs_Energy:public Oxs_Ext {
private:
  // Track count of number of times GetEnergy() has been
  // called in current problem run.
  UINT4m calc_count;
#ifdef EXPORT_CALC_COUNT
  // Make calc_count available for output.
  Oxs_ScalarOutput<Oxs_Energy> calc_count_output;
  void FillCalcCountOutput(const Oxs_SimState&);
#endif // EXPORT_CALC_COUNT

#if REPORT_TIME
  // energytime records time (cpu and wall) spent in the GetEnergyData
  // member function.  The results are written to stderr when this
  // object is destroyed or re-initialized.
  // Note: Timing errors may occur if an exception is thrown from inside
  //       GetEnergyData, because there is currently no attempt made to
  //       catch such exceptions and stop the stopwatch.  This could be
  //       done, but it probably isn't necessary for a facility which
  //       is intended only for development purposes.
  Nb_StopWatch energytime;
#endif // REPORT_TIME

  void SetupOutputs(); // Utility routine called by constructors.

  // Expressly disable default constructor, copy constructor and
  // assignment operator by declaring them without defining them.
  Oxs_Energy();
  Oxs_Energy(const Oxs_Energy&);
  Oxs_Energy& operator=(const Oxs_Energy&);

protected:

  Oxs_Energy(const char* name,      // Child instance id
             Oxs_Director* newdtr); // App director
  Oxs_Energy(const char* name,
             Oxs_Director* newdtr,
	     const char* argstr);   // MIF block argument string

  // Standard outputs presented by all energy objects.  These are
  // conceptually public, but are specified private to force clients
  // to use the Oxs_Director output_map interface.
  void UpdateStandardOutputs(const Oxs_SimState&);
  Oxs_ScalarOutput<Oxs_Energy> energy_sum_output;
  Oxs_VectorFieldOutput<Oxs_Energy> field_output;

  // Energy and energy derivatives calculation function.  This
  // should be "conceptually const."  The energy return is average
  // energy density for the corresponding cell, in J/m^3.  The
  // field is in A/m, pE_pt is in J/s.
  //   There are two GetEnergy functions, GetEnergy and GetEnergyData.
  // The first is a private, virtual member function of Oxs_Energy.
  // It takes as imports Oxs_MeshValue references into which to store
  // the energy and H results.  The second is a non-virtual public
  // member function that passes in Oxs_MeshValue references that
  // *may* be used to store the results, and separate parameters
  // that return pointers to where the results actually were stored.
  // If output caching in enabled, the Oxs_Energy base class method
  // GetEnergyData calls the virtual GetEnergy function with references
  // to the output cache Oxs_MeshValue objects.  Otherwise the import
  // buffer space is used.
  //   NOTE : The pE_pt export is the partial derivative of energy
  // with respect to time.  For most energy terms this will be
  // 0.  It will only be non-zero if there is an explicit dependence
  // on time, as for example with a time-varying applied field.
  virtual void GetEnergy(const Oxs_SimState& state,
			 Oxs_EnergyData& oed) const =0;

public:

  virtual ~Oxs_Energy();

  // Default problem initializer routine.  This sets up default
  // output objects, so any child that redefines this function
  // should embed a call to this Init() inside the child
  // specific version.
  virtual BOOL Init();

  void GetEnergyData(const Oxs_SimState& state,Oxs_EnergyData& oed);
  //   NOTE : The energy_buffer and field_buffer pointers in oed
  // must point to valid Oxs_MeshValue objects.  An exception will
  // be thrown if either is a null pointer.  However, GetEnergyData
  // will resize the buffers as necessary, so empty, 0-sized
  // Oxs_MeshValue objects may be passed in.  This also means that
  // allocated memory inside these objects may change as a side
  // effect of the call.
};


#endif // _OXS_ENERGY
