/* FILE: oxsexcept.h                 -*-Mode: c++-*-
 *
 * Exception classes for Oxs.
 *
 */

#ifndef _OXS_EXCEPT
#define _OXS_EXCEPT

#include "oc.h"
#include <string>

OC_USE_STD_NAMESPACE;  // Specify std namespace, if supported
OC_USE_STRING;

/* End includes */

// Dummy declarations that should move over to oommf/pkg/oc
#define OXS_THROW(except,text)                                     \
   throw except(String("ERROR\n"                                   \
		       "   in file: " __FILE__ "\n"                \
		       "   at line: " OC_STRINGIFY(__LINE__) "\n") \
		+ text );

#define OXS_TCLTHROW(text,ei,ec)                                   \
   throw Oxs_TclError(String("ERROR\n"                             \
		       "   in file: " __FILE__ "\n"                \
		       "   at line: " OC_STRINGIFY(__LINE__) "\n") \
		      + text, ei, ec);


// Oxs exceptions
class Oxs_Exception {
private:
  String msg;
public:
  Oxs_Exception(const String& text);
  String MessageText() { return msg; }
  virtual const char* MessageType() =0;
  virtual ~Oxs_Exception() {}
  void Prepend(const String& prefix) { msg = prefix + msg; }
  void Postpend(const String& suffix) { msg += suffix; }
};

// Bad pointer
class Oxs_BadPointer : public Oxs_Exception {
public:
  const char* MessageType() { return "Bad pointer"; }
  Oxs_BadPointer(const String& text) : Oxs_Exception(text) {}
};

// Data structure lock action failure
class Oxs_BadLock : public Oxs_Exception {
public:
  const char* MessageType() { return "Bad lock"; }
  Oxs_BadLock(const String& text) : Oxs_Exception(text) {}
};

// Data structure index invalid
class Oxs_BadIndex : public Oxs_Exception {
public:
  const char* MessageType() { return "Bad data index"; }
  Oxs_BadIndex(const String& text) : Oxs_Exception(text) {}
};

// Resource allocation failure
class Oxs_BadResourceAlloc : public Oxs_Exception {
public:
  const char* MessageType() { return "Resource allocation failure"; }
  Oxs_BadResourceAlloc(const String& text) : Oxs_Exception(text) {}
};

// Resource deallocation failure
class Oxs_BadResourceDealloc : public Oxs_Exception {
public:
  const char* MessageType() { return "Resource deallocation failure"; }
  Oxs_BadResourceDealloc(const String& text) : Oxs_Exception(text) {}
};

// Overflow detected
class Oxs_Overflow : public Oxs_Exception {
public:
  const char* MessageType() { return "Overflow"; }
  Oxs_Overflow(const String& text) : Oxs_Exception(text) {}
};

// Bad parameter
class Oxs_BadParameter : public Oxs_Exception {
public:
  const char* MessageType() { return "Bad parameter"; }
  Oxs_BadParameter(const String& text) : Oxs_Exception(text) {}
};

// Bad user input
class Oxs_BadUserInput : public Oxs_Exception {
public:
  const char* MessageType() { return "Bad user input"; }
  Oxs_BadUserInput(const String& text) : Oxs_Exception(text) {}
};

// Bad data
class Oxs_BadData : public Oxs_Exception {
public:
  const char* MessageType() { return "Bad data"; }
  Oxs_BadData(const String& text) : Oxs_Exception(text) {}
};

// Incomplete initialization
class Oxs_IncompleteInitialization : public Oxs_Exception {
public:
  const char* MessageType() {
    return "Incomplete or incorrect initialization";
  }
  Oxs_IncompleteInitialization(const String& text)
    : Oxs_Exception(text) {}
};

// Program logic error
class Oxs_ProgramLogicError : public Oxs_Exception {
public:
  const char* MessageType() { return "Program logic error"; }
  Oxs_ProgramLogicError(const String& text) : Oxs_Exception(text) {}
};

// No memory.  This exists for those broken C++ compilers (you
// know who you are) that don't throw a bad_alloc from new
class Oxs_NoMem : public Oxs_Exception {
public:
  const char* MessageType() { return "Insufficient memory"; }
  Oxs_NoMem(const String& text) : Oxs_Exception(text) {}
};

// Bad Tcl return type
class Oxs_TclBadReturnType : public Oxs_Exception {
public:
  const char* MessageType() { return "Wrong return type from Tcl script"; }
  Oxs_TclBadReturnType(const String& text) : Oxs_Exception(text) {}
};

// Error reported by Tcl interpreter
class Oxs_TclError : public Oxs_Exception {
private:
  String error_info;
  String error_code;
public:
  const char* ErrorInfo() { return error_info.c_str(); }
  const char* ErrorCode() { return error_code.c_str(); }
  const char* MessageType() { return "Tcl error"; }
  Oxs_TclError(const String& text,const String& ei,const String& ec)
    : Oxs_Exception(text), error_info(ei), error_code(ec) {}

};

#endif // _OXS_EXCEPT
