/* FILE: exchange6ngbr.cc            -*-Mode: c++-*-
 *
 * 6 neighbor exchange energy on rectangular mesh,
 * derived from Oxs_Energy class.
 *
 */

#include <string>

#include "atlas.h"
#include "nb.h"
#include "key.h"
#include "director.h"
#include "mesh.h"
#include "meshvalue.h"
#include "simstate.h"
#include "threevector.h"
#include "rectangularmesh.h"
#include "exchange6ngbr.h"
#include "energy.h"		// Needed to make MSVC++ 5 happy

OC_USE_STRING;

// Oxs_Ext registration support
OXS_EXT_REGISTER(Oxs_Exchange6Ngbr);

/* End includes */


// Constructor
Oxs_Exchange6Ngbr::Oxs_Exchange6Ngbr(
  const char* name,     // Child instance id
  Oxs_Director* newdtr, // App director
  const char* argstr)   // MIF input block parameters
  : Oxs_Energy(name,newdtr,argstr),
    coef_size(0), coef(NULL), mesh_id(0)
{
  // Process arguments
  OXS_GET_INIT_EXT_OBJECT("atlas",Oxs_Atlas,atlas);
  atlaskey.Set(atlas.GetPtr());
  /// Dependency lock is held until *this is deleted.

  // Allocate A matrix.  Because raw pointers are used, a memory
  // leak will occur if an exception is thrown inside this constructor.
  coef_size = atlas->GetRegionCount();
  if(coef_size<1) {
    String msg = String("Oxs_Atlas object ")
      + atlas->InstanceName()
      + String(" must contain at least one region.");

    throw Oxs_Ext::Error(msg.c_str());
  }
  coef = new REAL8m*[coef_size];
  coef[0] = new REAL8m[coef_size*coef_size];
  UINT4m i;
  for(i=1;i<coef_size;i++) coef[i] = coef[i-1] + coef_size;

  // Fill coef matrix
  String typestr;
  vector<String> params;
  REAL8m default_coef = 0.0;
  BOOL has_A = HasInitValue("A");
  BOOL has_lex = HasInitValue("lex");
  if(has_A && has_lex) {
    throw Oxs_Ext::Error(this,"Invalid exchange coefficient request:"
			 " both A and lex specified; only one should"
			 " be given.");
  } else if(has_lex) {
    excoeftype = LEX_TYPE;
    typestr = "lex";
    default_coef = GetRealInitValue("default_lex",0.0);
    FindRequiredInitValue("lex",params);
  } else {
    excoeftype = A_TYPE;
    typestr = "A";
    default_coef = GetRealInitValue("default_A",0.0);
    FindRequiredInitValue("A",params);
  }

  // Fill A matrix
  for(i=0;i<coef_size*coef_size;i++) coef[0][i] = default_coef;
  if(params.empty()) {
    String msg = String("Empty parameter list for key \"")
      + typestr + String("\"");
    throw Oxs_Ext::Error(this,msg);
  }
  if(params.size()%3!=0) {
      char buf[512];
      Oc_Snprintf(buf,sizeof(buf),
		  "Number of elements in %s sub-list must be"
		  " divisible by 3"
		  " (actual sub-list size: %u)",
		  typestr.c_str(),(unsigned int)params.size());
      throw Oxs_Ext::Error(this,buf);
  }
  for(i=0;i<params.size();i+=3) {
    INT4m i1 = atlas->GetRegionId(params[i]);
    INT4m i2 = atlas->GetRegionId(params[i+1]);
    if(i1<0 || i2<0) {
      char buf[4096];
      char* cptr=buf;
      if(i1<0) {
	Oc_Snprintf(buf,sizeof(buf),
		    "First entry in %s[%u] sub-list, \"%s\","
		    " is not a known region in atlas \"%s\".  ",
		    typestr.c_str(),i/3,params[i].c_str(),
                    atlas->InstanceName());
	cptr += strlen(buf);
      }
      if(i2<0) {
	Oc_Snprintf(cptr,sizeof(buf)-(cptr-buf),
		    "Second entry in %s[%u] sub-list, \"%s\","
		    " is not a known region in atlas \"%s\".  ",
		    typestr.c_str(),i/3,params[i+1].c_str(),
                    atlas->InstanceName());
      }
      String msg = String(buf);
      msg += String("Known regions:");
      vector<String> regions;
      atlas->GetRegionList(regions);
      for(unsigned int j=0;j<regions.size();++j) {
	msg += String(" \n");
	msg += regions[j];
      }
      throw Oxs_Ext::Error(this,msg);
    }
    BOOL err;
    REAL8m coefpair = Nb_Atof(params[i+2].c_str(),err);
    if(err) {
      char buf[4096];
      Oc_Snprintf(buf,sizeof(buf),
		  "Third entry in %s[%u] sub-list, \"%s\","
		  " is not a valid floating point number.",
		  typestr.c_str(),i/3,params[i+2].c_str());
      throw Oxs_Ext::Error(this,buf);
    }
    coef[i1][i2]=coefpair;
    coef[i2][i1]=coefpair; // coef should be symmetric
  }
  DeleteInitValue(typestr);

  VerifyAllInitArgsUsed();
}

Oxs_Exchange6Ngbr::~Oxs_Exchange6Ngbr()
{
  if(coef_size>0 && coef!=NULL) {
    delete[] coef[0];
    delete[] coef;
  }
}

BOOL Oxs_Exchange6Ngbr::Init()
{
  mesh_id = 0;
  region_id.Release();
  return Oxs_Energy::Init();
}

void Oxs_Exchange6Ngbr::GetEnergyA
(const Oxs_SimState& state,
 Oxs_EnergyData& oed
 ) const
{
  const Oxs_MeshValue<ThreeVector>& spin = state.spin;
  const Oxs_MeshValue<REAL8m>& Ms_inverse = *(state.Ms_inverse);

  // Use supplied buffer space, and reflect that use in oed.
  oed.energy = oed.energy_buffer;
  oed.field = oed.field_buffer;
  Oxs_MeshValue<REAL8m>& energy = *oed.energy_buffer;
  Oxs_MeshValue<ThreeVector>& field = *oed.field_buffer;

  const Oxs_RectangularMesh* mesh
    = dynamic_cast<const Oxs_RectangularMesh*>(state.mesh);
  if(mesh==NULL) {
    String msg = String("Import mesh to Oxs_Exchange6Ngbr::GetEnergy()"
                        " routine of object ")
      + String(InstanceName())
      + String(" is not an Oxs_RectangularMesh object.");
    throw Oxs_Ext::Error(msg.c_str());
  }

  UINT4m xdim = mesh->DimX();
  UINT4m ydim = mesh->DimY();
  UINT4m zdim = mesh->DimZ();
  UINT4m xydim = xdim*ydim;

  REAL8m wgtx = -1.0/(mesh->EdgeLengthX()*mesh->EdgeLengthX());
  REAL8m wgty = -1.0/(mesh->EdgeLengthY()*mesh->EdgeLengthY());
  REAL8m wgtz = -1.0/(mesh->EdgeLengthZ()*mesh->EdgeLengthZ());

  REAL8m hcoef = -2/MU0;

  for(UINT4m z=0;z<zdim;z++) {
    for(UINT4m y=0;y<ydim;y++) {
      for(UINT4m x=0;x<xdim;x++) {
	UINT4m i = mesh->Index(x,y,z); // Get base linear address
	ThreeVector base = spin[i];
	REAL8m Msii = Ms_inverse[i];
	if(Msii == 0.0) {
	  energy[i]=0.0;
	  field[i].Set(0.,0.,0.);
	  continue;
	}
	REAL8m* Arow = coef[region_id[i]];
	ThreeVector sum(0.,0.,0.);
	if(z>0) {
	  UINT4m j = i-xydim;
	  REAL8m Apair = Arow[region_id[j]];
	  if(Ms_inverse[j]!=0.0) sum += Apair*wgtz*(spin[j] - base);
	}
	if(y>0) {
	  UINT4m j = i-xdim;
	  REAL8m Apair = Arow[region_id[j]];
	  if(Ms_inverse[j]!=0.0) sum += Apair*wgty*(spin[j] - base);
	}
	if(x>0) {
	  UINT4m j = i-1;
	  REAL8m Apair = Arow[region_id[j]];
	  if(Ms_inverse[j]!=0.0) sum += Apair*wgtx*(spin[j] - base);
	}
	if(x<xdim-1) {
	  UINT4m j = i+1;
	  REAL8m Apair = Arow[region_id[j]];
	  if(Ms_inverse[j]!=0.0) sum += Apair*wgtx*(spin[j] - base);
	}
	if(y<ydim-1) {
	  UINT4m j = i+xdim;
	  REAL8m Apair = Arow[region_id[j]];
	  if(Ms_inverse[j]!=0.0) sum += Apair*wgty*(spin[j] - base);
	}
	if(z<zdim-1) {
	  UINT4m j = i+xydim;
	  REAL8m Apair = Arow[region_id[j]];
	  if(Ms_inverse[j]!=0.0) sum += Apair*wgtz*(spin[j]- base);
	}
	field[i] = (hcoef*Msii) * sum;
	energy[i] = (sum * base);
      }
    }
  }
}


void Oxs_Exchange6Ngbr::GetEnergyLex
(const Oxs_SimState& state,
 Oxs_EnergyData& oed
 ) const
{
  const Oxs_MeshValue<ThreeVector>& spin = state.spin;
  const Oxs_MeshValue<REAL8m>& Ms = *(state.Ms);

  // Use supplied buffer space, and reflect that use in oed.
  oed.energy = oed.energy_buffer;
  oed.field = oed.field_buffer;
  Oxs_MeshValue<REAL8m>& energy = *oed.energy_buffer;
  Oxs_MeshValue<ThreeVector>& field = *oed.field_buffer;

  const Oxs_RectangularMesh* mesh
    = dynamic_cast<const Oxs_RectangularMesh*>(state.mesh);
  if(mesh==NULL) {
    String msg = String("Import mesh to Oxs_Exchange6Ngbr::GetEnergy()"
                        " routine of object ")
      + String(InstanceName())
      + String(" is not an Oxs_RectangularMesh object.");
    throw Oxs_Ext::Error(msg.c_str());
  }

  UINT4m xdim = mesh->DimX();
  UINT4m ydim = mesh->DimY();
  UINT4m zdim = mesh->DimZ();
  UINT4m xydim = xdim*ydim;

  REAL8m wgtx = 1.0/(mesh->EdgeLengthX()*mesh->EdgeLengthX());
  REAL8m wgty = 1.0/(mesh->EdgeLengthY()*mesh->EdgeLengthY());
  REAL8m wgtz = 1.0/(mesh->EdgeLengthZ()*mesh->EdgeLengthZ());

  for(UINT4m z=0;z<zdim;z++) {
    for(UINT4m y=0;y<ydim;y++) {
      for(UINT4m x=0;x<xdim;x++) {
	UINT4m i = mesh->Index(x,y,z); // Get base linear address

	if(0 == Ms[i]) {
	  energy[i]=0.0;
	  field[i].Set(0.,0.,0.);
	  continue;
	}
	const REAL8m emult = (-0.5*MU0)*Ms[i];

	ThreeVector base = spin[i];

	REAL8m* lexrow = coef[region_id[i]];
	ThreeVector sum(0.,0.,0.);
	if(z>0) {
	  UINT4m j = i-xydim;
	  REAL8m lexpair = lexrow[region_id[j]];
	  sum += Ms[j]*lexpair*lexpair*wgtz*(spin[j] - base);
	}
	if(y>0) {
	  UINT4m j = i-xdim;
	  REAL8m lexpair = lexrow[region_id[j]];
	  sum += Ms[j]*lexpair*lexpair*wgty*(spin[j] - base);
	}
	if(x>0) {
	  UINT4m j = i-1;
	  REAL8m lexpair = lexrow[region_id[j]];
	  sum += Ms[j]*lexpair*lexpair*wgtx*(spin[j] - base);
	}
	if(x<xdim-1) {
	  UINT4m j = i+1;
	  REAL8m lexpair = lexrow[region_id[j]];
	  sum += Ms[j]*lexpair*lexpair*wgtx*(spin[j] - base);
	}
	if(y<ydim-1) {
	  UINT4m j = i+xdim;
	  REAL8m lexpair = lexrow[region_id[j]];
	  sum += Ms[j]*lexpair*lexpair*wgty*(spin[j] - base);
	}
	if(z<zdim-1) {
	  UINT4m j = i+xydim;
	  REAL8m lexpair = lexrow[region_id[j]];
	  sum += Ms[j]*lexpair*lexpair*wgtz*(spin[j]- base);
	}
	field[i] = sum;
	energy[i] = emult * (sum * base);
      }
    }
  }
}


void Oxs_Exchange6Ngbr::GetEnergy
(const Oxs_SimState& state,
 Oxs_EnergyData& oed
 ) const
{
  // See if mesh and/or atlas has changed.
  if(mesh_id !=  state.mesh->Id() || !atlaskey.SameState()) {
    // Setup region mapping
    mesh_id = 0; // Safety
    UINT4m size = state.mesh->Size();
    region_id.AdjustSize(state.mesh);
    ThreeVector location;
    for(UINT4m i=0;i<size;i++) {
      state.mesh->Center(i,location);
      if((region_id[i] = atlas->GetRegionId(location))<0) {
	String msg = String("Import mesh to Oxs_Exchange6Ngbr::GetEnergy()"
                            " routine of object ")
          + String(InstanceName())
	  + String(" has points outside atlas ")
	  + String(atlas->InstanceName());
	throw Oxs_Ext::Error(msg.c_str());
      }
    }
    mesh_id = state.mesh->Id();
  }
  if(excoeftype == LEX_TYPE) {
    GetEnergyLex(state,oed);
  } else {
    GetEnergyA(state,oed);
  }
}
