/* FILE: cachescalarfield.cc     -*-Mode: c++-*-
 *
 * Cache holding a scalar field.
 * This class is derived from Oxs_ScalarField class.
 *
 */

#include "oc.h"
#include "nb.h"

#include "cachescalarfield.h"

// Oxs_Ext registration support
OXS_EXT_REGISTER(Oxs_CacheScalarField);

/* End includes */


// Constructor
Oxs_CacheScalarField::Oxs_CacheScalarField(
  const char* name,     // Child instance id
  Oxs_Director* newdtr, // App director
  const char* argstr)   // MIF input block parameters
  : Oxs_ScalarField(name,newdtr,argstr),
    mesh_id(0)
{
  // Process arguments
  REAL8m offset = GetRealInitValue("offset",0.0);
  REAL8m multiplier = GetRealInitValue("multiplier",1.0);

  Oxs_OwnedPointer<Oxs_ScalarField> field;
  OXS_GET_INIT_EXT_OBJECT("field",Oxs_ScalarField,field);

  OXS_GET_INIT_EXT_OBJECT("mesh",Oxs_Mesh,mesh);

  VerifyAllInitArgsUsed();

  // Fill cache
  field->FillMeshValue(mesh.GetPtr(),cache);
  const UINT4m size = cache.Size();
  for(UINT4m i=0;i<size;++i) {
    cache[i] = multiplier*cache[i] + offset;
  }
  mesh_id = mesh->Id();
}

REAL8m
Oxs_CacheScalarField::Value
(const ThreeVector& pt) const
{
#ifndef NDEBUG
  if(mesh_id != mesh->Id()) {
    String msg = String("Mesh change; invalid cache "
      "(Oxs_CacheScalarField::Value(const ThreeVector&) const)");
    throw Oxs_Ext::Error(msg.c_str());
  }
#endif
  UINT4m index = mesh->FindNearestIndex(pt);
  return cache[index];
}

void
Oxs_CacheScalarField::FillMeshValue
(const Oxs_Mesh* othermesh,
 Oxs_MeshValue<REAL8m>& array) const
{
  if(mesh_id != mesh->Id()) {
    String msg = String("Mesh change; invalid cache "
      "(Oxs_CacheScalarField::Value(const ThreeVector&) const)");
    throw Oxs_Ext::Error(msg.c_str());
  }
  if(othermesh->Id() == mesh_id) {
    // Same mesh; just copy values
    array = cache;
  } else {
    // Otherwise, use default method, which calls ::Value(), and hence
    // relies on mesh->FindNearestIndex()---which is considerably
    // slower.
    DefaultFillMeshValue(othermesh,array);
  }
}

void
Oxs_CacheScalarField::IncrMeshValue
(const Oxs_Mesh* othermesh,
 Oxs_MeshValue<REAL8m>& array) const
{
  if(mesh_id != mesh->Id()) {
    String msg = String("Mesh change; invalid cache "
      "(Oxs_CacheScalarField::IncrMeshValue() const)");
    throw Oxs_Ext::Error(msg.c_str());
  }
  if(othermesh->Id() == mesh_id) {
    // Same mesh; add values directly between meshes
    array += cache;
  } else {
    // Otherwise, use default method, which calls ::Value(), and hence
    // relies on mesh->FindNearestIndex()---which is considerably
    // slower.
    DefaultIncrMeshValue(othermesh,array);
  }
}

void
Oxs_CacheScalarField::MultMeshValue
(const Oxs_Mesh* othermesh,
 Oxs_MeshValue<REAL8m>& array) const
{
  if(mesh_id != mesh->Id()) {
    String msg = String("Mesh change; invalid cache "
      "(Oxs_CacheScalarField::MultMeshValue() const)");
    throw Oxs_Ext::Error(msg.c_str());
  }
  if(othermesh->Id() == mesh_id) {
    // Same mesh; multiply values directly between meshes
    array *= cache;
  } else {
    // Otherwise, use default method, which calls ::Value(), and hence
    // relies on mesh->FindNearestIndex()---which is considerably
    // slower.
    DefaultMultMeshValue(othermesh,array);
  }
}
