/* FILE: qdemag.h            -*-Mode: c++-*-
 *
 * Average H demag field across rectangular cells.  This is a modified
 * version of the simpledemag class, which uses symmetries in the
 * interaction coefficients to reduce memory usage.
 *
 */

#ifndef _OXS_QDEMAG
#define _OXS_QDEMAG

#include "energy.h"
#include "fft.h"
#include "key.h"
#include "mesh.h"
#include "meshvalue.h"
#include "simstate.h"
#include "threevector.h"
#include "rectangularmesh.h"

/* End includes */

class Oxs_QDemag:public Oxs_Energy {
private:
  mutable UINT4m rdimx;  // Natural size of real data
  mutable UINT4m rdimy;
  mutable UINT4m rdimz;
  mutable UINT4m cdimx; // Full size of complex data
  mutable UINT4m cdimy;
  mutable UINT4m cdimz;
  // 2*cdimx>=rdimx, cdimy>=rdimy, cdimz>=rdimz
  // cdim[xyz] should be powers of 2.
  mutable UINT4m cstridey; // Strides across complex data
  mutable UINT4m cstridez;
  // cstridey>=cdimx, cstridez>=cdimy*cstridey
  // The stride sizes for the real arrays are just double the
  // complex strides, except cstride1 and rstride1 are assumed
  // to be 1.  Total matrix size is effectively cdimz*cstridez
  // Oxs_Complex elements, or twice that many "double" elements.

  mutable UINT4m mesh_id;

  // The A## arrays hold demag coefficients, transformed into
  // frequency domain.  These are held long term.  xcomp,
  // ycomp, and zcomp are used as temporary space, first to hold
  // the transforms of Mx, My, and Mz, then to store Hx, Hy, and
  // Hz.
  mutable UINT4m adimx;    mutable UINT4m adimy;   mutable UINT4m adimz;
  mutable UINT4m astridey; mutable UINT4m astridez;
  mutable OXS_COMPLEX_REAL_TYPE *A00;
  mutable OXS_COMPLEX_REAL_TYPE *A01;
  mutable OXS_COMPLEX_REAL_TYPE *A02;
  mutable OXS_COMPLEX_REAL_TYPE *A11;
  mutable OXS_COMPLEX_REAL_TYPE *A12;
  mutable OXS_COMPLEX_REAL_TYPE *A22;
  mutable Oxs_Complex *xcomp;
  mutable Oxs_Complex *ycomp;
  mutable Oxs_Complex *zcomp;
  mutable Oxs_FFT3D fft; // All transforms are same size, so we need
  /// only one Oxs_FFT3D object.

  mutable Oxs_MeshValue<ThreeVector> local_field_buffer;
  mutable Oxs_MeshValue<REAL8m> local_energy_buffer;
  mutable OXS_COMPLEX_REAL_TYPE Nx; // Cell self-demag coefficients
  mutable OXS_COMPLEX_REAL_TYPE Ny;
  mutable OXS_COMPLEX_REAL_TYPE Nz;
  INT4m local_energy_skip_request;
  mutable INT4m local_energy_skip_count;

  void (Oxs_QDemag::*fillcoefs)(const Oxs_Mesh*) const;
  void FillCoefficientArraysFast(const Oxs_Mesh* mesh) const;
  void FillCoefficientArraysStandard(const Oxs_Mesh* mesh) const;
  /// The "standard" variant is simpler but slower, and is retained
  /// mainly for testing and development purposes.

  UINT4m NextPowerOfTwo(UINT4m n) const;  // Helper function
  void ReleaseMemory() const;

  void GetFullEnergy(const Oxs_SimState& state,
		     Oxs_EnergyData& oed) const;

protected:
  virtual void GetEnergy(const Oxs_SimState& state,
			 Oxs_EnergyData& oed) const;

public:
  virtual const char* ClassName() const; // ClassName() is
  /// automatically generated by the OXS_EXT_REGISTER macro.
  Oxs_QDemag(const char* name,     // Child instance id
	     Oxs_Director* newdtr, // App director
	     const char* argstr);  // MIF input block parameters
  virtual ~Oxs_QDemag() { ReleaseMemory(); }
  virtual BOOL Init();
};


#endif // _OXS_QDEMAG
